@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# Copyright (C) 2009-2022  Alex Schroeder <alex@gnu.org>
+# Copyright (C) 2009-2021  Alex Schroeder <alex@gnu.org>
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU Affero General Public License as published by the Free
@@ -16,7 +16,7 @@
 
 package Game::TextMapper;
 
-our $VERSION = 1.05;
+our $VERSION = 1.01;
 
 use Game::TextMapper::Log;
 use Game::TextMapper::Point;
@@ -33,6 +33,8 @@ use Game::TextMapper::Traveller;
 
 use Modern::Perl '2018';
 use Mojolicious::Lite;
+use Mojolicious::Command::render;
+use Mojolicious::Command::random;
 use Mojo::DOM;
 use Mojo::Util qw(url_escape xml_escape);
 use File::ShareDir 'dist_dir';
@@ -41,9 +43,6 @@ use Pod::Simple::Text;
 use List::Util qw(none);
 use Cwd;
 
-# Commands for the command line!
-push @{app->commands->namespaces}, 'Game::TextMapper::Command';
-
 # Change scheme if "X-Forwarded-Proto" header is set (presumably to HTTPS)
 app->hook(before_dispatch => sub {
   my $c = shift;
@@ -124,7 +123,7 @@ get '/random' => sub {
   my $bw = $c->param('bw');
   my $width = $c->param('width');
   my $height = $c->param('height');
-  $c->render(template => 'edit', map => Game::TextMapper::Smale->new->generate_map($width, $height, $bw));
+  $c->render(template => 'edit', map => Game::TextMapper::Smale::generate_map($bw, $width, $height));
 };
 
 get '/smale' => sub {
@@ -133,10 +132,10 @@ get '/smale' => sub {
   my $width = $c->param('width');
   my $height = $c->param('height');
   if ($c->stash('format')||'' eq 'txt') {
-    $c->render(text => Game::TextMapper::Smale->new->generate_map($width, $height));
+    $c->render(text => Game::TextMapper::Smale::generate_map(undef, $width, $height));
   } else {
     $c->render(template => 'edit',
-	       map => Game::TextMapper::Smale->new->generate_map($width, $height, $bw));
+	       map => Game::TextMapper::Smale::generate_map($bw, $width, $height));
   }
 };
 
@@ -145,7 +144,7 @@ get '/smale/random' => sub {
   my $bw = $c->param('bw');
   my $width = $c->param('width');
   my $height = $c->param('height');
-  my $map = Game::TextMapper::Smale->new->generate_map($width, $height, $bw);
+  my $map = Game::TextMapper::Smale::generate_map($bw, $width, $height);
   my $svg = Game::TextMapper::Mapper::Hex->new(dist_dir => $dist_dir)
       ->initialize($map)
       ->svg();
@@ -157,7 +156,7 @@ get '/smale/random/text' => sub {
   my $bw = $c->param('bw');
   my $width = $c->param('width');
   my $height = $c->param('height');
-  my $text = Game::TextMapper::Smale->new->generate_map($width, $height, $bw);
+  my $text = Game::TextMapper::Smale::generate_map($bw, $width, $height);
   $c->render(text => $text, format => 'txt');
 };
 
@@ -177,7 +176,6 @@ sub alpine_map {
 		$c->param('bump'),
 		$c->param('bottom'),
 		$c->param('arid'),
-		$c->param('wind'),
 		$seed,
 		$url,
 		$step,
@@ -231,11 +229,8 @@ get '/alpine/document' => sub {
   my $type = $c->param('type') || 'hex';
   # use the same seed for all the calls
   my $seed = $c->param('seed');
-  if (not defined $seed) {
-    $seed = int(rand(1000000000));
-    $c->param('seed' => $seed);
-  }
-  for my $step (1 .. 18) {
+  $seed = $c->param('seed' => int(rand(1000000000))) unless defined $seed;
+  for my $step (1 .. 16) {
     my $map = alpine_map($c, $step);
     my $mapper;
     if ($type eq 'hex') {
@@ -249,8 +244,8 @@ get '/alpine/document' => sub {
   };
   $c->stash("maps" => \@maps);
 
-  # The documentation needs all the defaults of Alpine::generate_map (but
-  # we'd like to use a smaller map because it is so slow).
+  # the documentation needs all the defaults of Alpine::generate_map (but
+  # we'd like to use a smaller map because it is so slow)
   my $width = $c->param('width') // 20;
   my $height = $c->param('height') // 5; # instead of 10
   my $steepness = $c->param('steepness') // 3;
@@ -261,7 +256,6 @@ get '/alpine/document' => sub {
   my $bottom = $c->param('bottom') // 0;
   my $arid = $c->param('arid') // 2;
 
-  # Generate the documentation text based on the stashed maps.
   $c->render(template => 'alpine_document',
 	     seed => $seed,
 	     width => $width,
@@ -529,8 +523,7 @@ sub apocalypse_map {
   my $c = shift;
   my $seed = $c->param('seed') || int(rand(1000000000));
   srand($seed);
-  my $hash = $c->req->params->to_hash;
-  return Game::TextMapper::Apocalypse->new(%$hash)
+  return Game::TextMapper::Apocalypse->new()
       ->generate_map();
 }
 
@@ -562,8 +555,9 @@ sub star_map {
   my $c = shift;
   my $seed = $c->param('seed') || int(rand(1000000000));
   srand($seed);
-  my $hash = $c->req->params->to_hash;
-  return Game::TextMapper::Traveller->new(%$hash)->generate_map();
+  return Game::TextMapper::Traveller
+      ->with_roles('Game::TextMapper::Schroeder::Hex')->new()
+      ->generate_map();
 }
 
 get '/traveller' => sub {
@@ -590,6 +584,13 @@ get '/traveller/random/text' => sub {
   $c->render(text => $map, format => 'txt');
 };
 
+get '/source' => sub {
+  my $c = shift;
+  seek(DATA,0,0);
+  local $/ = undef;
+  $c->render(text => <DATA>, format => 'txt');
+};
+
 get '/help' => sub {
   my $c = shift;
 
@@ -634,12 +635,6 @@ Game::TextMapper - a web app to generate maps based on text files
 The script parses a text description of a hex map and produces SVG output. Use
 your browser to view SVG files and use Inkscape to edit them.
 
-=head2 Tutorial
-
-Note that if you look at the help page
-L<online|https://campaignwiki.org/text-mapper/help> there are links to run all
-these examples.
-
 Here's a small example:
 
     grass attributes fill="green"
@@ -893,7 +888,8 @@ directory.
     0302 sand
     0303 sand
 
-=head3 The default library
+
+=head3 Default library
 
 Source of the map:
 L<http://themetalearth.blogspot.ch/2011/03/opd-entry.html>
@@ -940,6 +936,7 @@ L<https://campaignwiki.org/contrib/gridmapper.txt>
 Result:
 L<https://campaignwiki.org/text-mapper?type=square&map=include+gridmapper-example.txt>
 
+
 =head2 Large Areas
 
 If you want to surround a piece of land with a round shore line, a
@@ -1071,82 +1068,45 @@ this into account when putting a license onto the map. Thus, if your map does
 not include the hex 0101, you can't use coordinates for the license text around
 the origin at (0,0) – you'll have to move it around.
 
-=head3 Smale
+=head2 Random
 
-The default algorithm was developed by Erin D. Smale. See L<Hex-based Campaign
-Design (Part 1)|http://www.welshpiper.com/hex-based-campaign-design-part-1/> and
-L<Hex-based Campaign Design (Part
-2)|http://www.welshpiper.com/hex-based-campaign-design-part-2/> for more
-information.
+The Random button generates a random landscape based on the algorithm
+developed by Erin D. Smale. See
+L<http://www.welshpiper.com/hex-based-campaign-design-part-1/> and
+L<http://www.welshpiper.com/hex-based-campaign-design-part-2/> for
+more information. The output uses the I<Gnomeyland> icons by Gregory
+B. MacKenzie. These are licensed under the Creative Commons
+Attribution-ShareAlike 3.0 Unported License. To view a copy of this
+license, visit L<http://creativecommons.org/licenses/by-sa/3.0/>.
 
-The output uses the I<Gnomeyland> icons by Gregory B. MacKenzie. These are
-licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported License.
-To view a copy of this license, visit
-L<http://creativecommons.org/licenses/by-sa/3.0/>. If you use these maps in your
-works, you must take this into account.
+If you're curious: (11,11) is the starting hex.
 
-See L<Game::TextMapper::Smale> for more information.
+=head2 Alpine
 
-=head3 Alpine
+The Alpine button generates a random landscape based on an algorithm developed
+by Alex Schroeder. The output also uses the I<Gnomeyland> icons by Gregory B.
+MacKenzie. These are licensed under the Creative Commons Attribution-ShareAlike
+3.0 Unported License. To view a copy of this license, visit
+L<http://creativecommons.org/licenses/by-sa/3.0/>.
 
-The Alpine algorithm was developed by Alex Schroeder. See L<Alpine map generator
-1|https://alexschroeder.ch/wiki/2016-08-06_Alpine_Map_Generator> and L<Alpine
-map generator 2|https://alexschroeder.ch/wiki/2016-08-16_Alpine_Map_Generator>
-for more information.
+=head2 Gridmapper
 
-The output also uses the I<Gnomeyland> icons by Gregory B. MacKenzie. These are
-licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported License.
-To view a copy of this license, visit
-L<http://creativecommons.org/licenses/by-sa/3.0/>. If you use these maps in your
-works, you must take this into account.
+The Gridmapper button generates a random mini-dungeon based on the algorithm by
+Alex Schroeder and based on geomorph sketches by Robin Green.
 
-See L<Game::TextMapper::Schroeder::Alpine> for more information.
+=head2 Islands
 
-=head3 Apocalypse
+The Island links generate a random landscape based on the algorithm by Alex
+Schroeder. The output also uses the I<Gnomeyland> icons by Gregory B. MacKenzie.
+These are licensed under the Creative Commons Attribution-ShareAlike 3.0
+Unported License. To view a copy of this license, visit
+L<http://creativecommons.org/licenses/by-sa/3.0/>.
 
-The Alpine algorithm was developed by Alex Schroeder. See L<Hex describing the
-post-apocalypse|https://alexschroeder.ch/wiki/2020-10-02_Hex_describing_the_post-apocalypse>
-for more information.
-
-The output uses the default library. This library is dedicated to the public.
-domain.
-
-See L<Game::TextMapper::Schroeder::Alpine> for more information.
-
-=head3 Gridmapper
-
-The Gridmapper algorithm was developed by Alex Schroeder and is based on
-geomorph sketches by Robin Green. See L<The Nine Forms of the Five Room
-Dungeon|https://gnomestew.com/the-nine-forms-of-the-five-room-dungeon/> by
-Matthew J. Neagley for more information.
-
-The output uses the Dungeons library. This library is dedicated to the public
-domain.
-
-See L<Game::TextMapper::Gridmapper> for more information.
-
-=head3 Islands
-
-The Island algorithm was developed by Alex Schroeder. See
-L<https://alexschroeder.ch/wiki/2020-04-25_Island_generator_using_J> and
-L<https://alexschroeder.ch/wiki/2020-05-01_Island_map_generator_and_Text_Mapper>
-for more information.
-
-The output also uses the I<Gnomeyland> icons by Gregory B. MacKenzie. These are
-licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported License.
-To view a copy of this license, visit
-L<http://creativecommons.org/licenses/by-sa/3.0/>. If you use these maps in your
-works, you must take this into account.
-
-See L<Game::TextMapper::Schroeder::Islands> for more information.
-
-=head3 Traveller
+=head2 Traveller
 
 The Traveller link generates a random landscape based on Classic Traveller with
 additions by Vicky Radcliffe and Alex Schroeder.
 
-See L<Game::TextMapper::Traveller> for more information.
-
 =head2 Border Adjustments
 
 The border adjustments can be a little unintuitive. Let's assume the default map
@@ -1331,19 +1291,23 @@ library will access the files installed with the entire distribution.
 You can call the script from the command line. The B<render> command reads a map
 description from STDIN and prints it to STDOUT.
 
-    text-mapper render < contrib/forgotten-depths.txt > forgotten-depths.svg
-
-See L<Game::TextMapper::Command::render> for more.
+    perl text-mapper.pl render < contrib/forgotten-depths.txt > forgotten-depths.svg
 
 The B<random> command prints a random map description to STDOUT.
 
-    text-mapper random > map.txt
-
-See L<Game::TextMapper::Command::random> for more.
+    perl text-mapper.pl random > map.txt
 
 Thus, you can pipe the random map in order to render it:
 
-    text-mapper random | text-mapper render > map.svg
+    perl text-mapper.pl random | perl text-mapper.pl render > map.svg
+
+You can read this documentation in a text terminal, too:
+
+    pod2text text-mapper.pl
+
+Alternatively:
+
+    perl text-mapper.pl get /help | w3m -T text/html
 
 =cut
 
@@ -1394,7 +1358,7 @@ for an explanation of what these parameters do.
 <hr>
 <p>
 <%= link_to smale => begin %>Random<% end %>
-generates map data based on Erin D. Smale's <em>Hex-Based Campaign Design</em>
+will generate map data based on Erin D. Smale's <em>Hex-Based Campaign Design</em>
 (<a href="http://www.welshpiper.com/hex-based-campaign-design-part-1/">Part 1</a>,
 <a href="http://www.welshpiper.com/hex-based-campaign-design-part-2/">Part 2</a>).
 You can also generate a random map
@@ -1416,7 +1380,7 @@ You'll find the map description in a comment within the SVG file.
 
 <hr>
 <p>
-<%= link_to alpine => begin %>Alpine<% end %> generates map data based on Alex
+<%= link_to alpine => begin %>Alpine<% end %> will generate map data based on Alex
 Schroeder's algorithm that's trying to recreate a medieval Swiss landscape, with
 no info to back it up, whatsoever. See it
 <%= link_to url_for('alpinedocument')->query(height => 5) => begin %>documented<% end %>.
@@ -1463,7 +1427,7 @@ explanation of what these parameters do.
 <hr>
 <p>
 <%= link_to url_for('gridmapper')->query(type => 'square') => begin %>Gridmapper<% end %>
-generates dungeon map data based on geomorph sketches by Robin Green. Or
+will generate dungeon map data based on geomorph sketches by Robin Green. Or
 just keep reloading one of these links:
 <%= link_to url_for('gridmapperrandom')->query(rooms => 5) => begin %>5 rooms<% end %>,
 <%= link_to url_for('gridmapperrandom')->query(rooms => 10) => begin %>10 rooms<% end %>,
@@ -1491,47 +1455,15 @@ Just caves
 
 <hr>
 
+<p>Ideas and work in progress…
+
 <p><%= link_to url_for('apocalypse') => begin %>Apocalypse<% end %> generates a post-apocalyptic map.
 <%= link_to url_for('apocalypserandom') => begin %>Reload<% end %> for lots of post-apocalyptic maps.
 You'll find the map description in a comment within the SVG file.
-%= form_for apocalypse => begin
-<p>
-<table>
-<tr><td>Width:</td><td>
-%= number_field cols => 20, min => 1
-</td><td>Height:</td><td>
-%= number_field rows => 10, min => 1
-</td></tr>
-<tr><td>Region Size:</td><td>
-%= number_field region_size => 5, min => 1
-</td><td>Settlement Chance:</td><td>
-%= number_field settlement_chance => 0.1, min => 0, max => 1, step => 0.1
-</td></tr></table>
-<p>
-%= submit_button "Generate Map Data"
-% end
-
-<hr>
 
 <p><%= link_to url_for('traveller') => begin %>Traveller<% end %> generates a star map.
 <%= link_to url_for('travellerrandom') => begin %>Reload<% end %> for lots of random star maps.
 You'll find the map description in a comment within the SVG file.
-%= form_for traveller => begin
-<p>
-<table>
-<tr><td>Width:</td><td>
-%= number_field cols => 8, min => 1
-</td></tr>
-<tr><td>Height:</td><td>
-%= number_field rows => 10, min => 1
-</td></tr></table>
-<p>
-%= submit_button "Generate Map Data"
-% end
-
-<hr>
-
-<p>Ideas and work in progress…
 
 <p><%= link_to url_for('island') => begin %>Island<% end %> generates a hotspot-inspired island chain.
 Reload <%= link_to url_for('islandrandom') => begin %>Hex Island<% end %>
@@ -1724,15 +1656,6 @@ It is insufficiently drained. We use grey swamps to indicate this.</p>
 
 %== shift(@$maps)
 
-<p>We determined the predominant wind direction (see purple arrow in 01.01) and
-mark areas in the wind shadow of mountains as having no river sources,
-presumably because of reduced rainfall. Specifically, a hex with altitude 7 or 8
-next to a hex where the wind is coming from that is at the same altitude or
-higher is marked as "dry". The only effect is that there can be no river sources
-in these hexes (as these are all at altitudes 7 and 8)</p>
-
-%== shift(@$maps)
-
 <p>We add a river sources high up in the mountains (altitudes 7 and 8), merging
 them as appropriate. These rivers flow as indicated by the arrows. If the river
 source is not a mountain (altitude 8) or a bog (altitude 7), then we place a
@@ -1757,31 +1680,19 @@ otherwise dark-grey.</p>
 <p>Wherever there is water and no swamp, forests will form. The exact type again
 depends on the altitude: light green fir-forest (altitude 7 and higher), green
 fir-forest (altitude 6), green forest (altitude 4–5), dark-green forest
-(altitude 3 and lower). Once a forest is placed, it expands up to <%= $arid %>
-hexes away, even if those hexes have no water flowing through them. When
-considering neighbouring hexes, they have to be at the same altitude or lower;
-when considering hexes with an intermediary hex, the intermediary hex has to be
-at the same altitude or lower, and the other he has to be at the same altitude
-or lower as the intermediary hex.</p>
-
-<p>You probably need fewer peaks on your map to verify this (a <%= link_to
-url_with('alpinedocument')->query({peaks => 1}) => begin %>lonely mountain<% end
+(altitude 3 and lower). Once a forest is placed, it expands up to <%= $arid %> hexes
+away, even if those hexes have no water flowing through them. You probably need
+fewer peaks on your map to verify this (a <%= link_to
+url_with('alpinerandom')->query({peaks => 1}) => begin %>lonely mountain<% end
 %> map, for example).</p>
 
 %== shift(@$maps)
 
 <p>Any remaining hexes have no water nearby and are considered to be little more
-arid. At high altitudes, they get "light-grey grass"; at lower altitudes they
-get "light-green bushes". For these lower altitude badlands, we add more variety
-by simulating areas where conditions are bad. We pick a quarter of these hexes,
-and deteriorate them, and their immediate neighbours. That is, we take little
-"circles" of seven hexes each, and place them in these areas. Whenever they
-overlap, conditions deteriorate even further: light-green bushes → light-green
-grass → dust grass → dust hill → dust desert.</p>
-
-<p>You probably need fewer peaks on your map to verify this (a <%= link_to
-url_with('alpinedocument')->query({peaks => 1}) => begin %>lonely mountain<% end
-%> map, for example).</p>
+arid. They get bushes, a hill (20% of the time at altitudes 3 or higher), or
+some grass (60% of the time at altitudes 3 and lower). Higher up, these are
+light grey (altitude 6–7), otherwise they are light green (altitude 5 and
+below).</p>
 
 %== shift(@$maps)
 
@@ -1790,44 +1701,23 @@ url_with('alpinedocument')->query({peaks => 1}) => begin %>lonely mountain<% end
 %== shift(@$maps)
 
 <p>Wherenver there is forest, settlements will be built. These reduce the
-density of the forest. Of the settlements are big, trees disappear completely
-and are replaced by soil. If the village or town is next to a water hex, it gets
-a port and grows by one step. Note that currently it's impossibleto get a
-city.</p>
+density of the forest. There are three levels of settlements: thorps, villages
+and towns.</p>
 
 <table>
-<tr><th>Settlement</th><th>Conditions</th><th>Change to</th><th>Number</th>
-    <th>Minimum Distance</th><th>Near water</th></tr>
-<tr><td>thorp</td><td>fir-forest, or forest</td><td>firs, or trees</td><td class="numeric">10%</td>
-    <td class="numeric">2</td><td>unchanged</td></tr>
-<tr><td>village</td><td>forest &amp; river</td><td>trees</td><td class="numeric">5%</td>
-    <td class="numeric">5</td><td>town &amp; port</td></tr>
-<tr><td>town</td><td>forest or dark-forest &amp; river</td><td>soil</td><td class="numeric">2½%</td>
-    <td class="numeric">10</td><td>large town &amp; port</td></tr>
-<tr><td>large town</td><td>none</td><td>light soil</td><td class="numeric">0%</td>
-    <td class="numeric">n/a</td><td>city &amp; port</td></tr>
-<tr><td>city</td><td>none</td><td>light soil</td><td class="numeric">0%</td>
-    <td class="numeric">n/a</td><td>unchanged</td></tr>
-<tr><td>law</td><td>white mountain</td><td>unchanged</td><td class="numeric">2½%</td>
-    <td class="numeric">10</td><td>unchanged</td></tr>
-<tr><td>chaos</td><td>swamp</td><td>unchanged</td><td class="numeric">2½%</td>
-    <td class="numeric">10</td><td>unchanged</td></tr>
+<tr><th>Settlement</th><th>Forest</th><th>Number</th><th>Minimum Distance</th></tr>
+<tr><td>Thorp</td><td>fir-forest, forest</td><td class="numeric">10%</td><td class="numeric">2</td></tr>
+<tr><td>Village</td><td>forest &amp; river</td><td class="numeric">5%</td><td class="numeric">5</td></tr>
+<tr><td>Town</td><td>forest &amp; river</td><td class="numeric">2½%</td><td class="numeric">10</td></tr>
+<tr><td>Law</td><td>white mountain</td><td class="numeric">2½%</td><td class="numeric">10</td></tr>
+<tr><td>Chaos</td><td>swamp</td><td class="numeric">2½%</td><td class="numeric">10</td></tr>
 </table>
 
 %== shift(@$maps)
 
 <p>Trails connect every settlement to any neighbor that is one or two hexes
 away. If no such neighbor can be found, we try to find neighbors that are three
-hexes away. If there are multiple options, we prefer the one at a lower
-altitude.</p>
-
-%== shift(@$maps)
-
-<p>Finally, we take advantage of the fact that rivers continue into the ocean.
-We identify river mouths where the altitude change is just 1 (i.e. no cliff) and
-extend the land into the water using a blue-green swamp. These are coastal
-marshes. We also check the next hex along the (invisible) river to check if this
-an ocean hex. If it is, we change it to water.
+hexes away.</p>
 
 %== shift(@$maps)
 
@@ -1866,7 +1756,8 @@ td, th {
 <p>
 <a href="https://campaignwiki.org/text-mapper">Text Mapper</a>&#x2003;
 <%= link_to 'Help' => 'help' %>&#x2003;
-<a href="https://alexschroeder.ch/cgit/text-mapper/about/">Git</a>&#x2003;
+<%= link_to 'Source' => 'source' %>&#x2003;
+<a href="https://alexschroeder.ch/cgit/hex-mapping/about/#text-mapper">Git</a>&#x2003;
 <a href="https://alexschroeder.ch/wiki/Contact">Alex Schroeder</a>
 </body>
 </html>
