@@ -19,19 +19,6 @@
 
 Game::TextMapper::Mapper - a text map parser and builder
 
-=head1 SYNOPSIS
-
-    use Modern::Perl;
-    use Game::TextMapper::Mapper::Hex;
-    my $map = <<EOT;
-    0101 forest
-    include default.txt
-    EOT
-    my $svg = Game::TextMapper::Mapper::Hex->new(dist_dir => 'share')
-      ->initialize($map)
-      ->svg();
-    print $svg;
-
 =head1 DESCRIPTION
 
 This class knows how to parse a text containing a map description into SVG
@@ -39,8 +26,12 @@ definitions, and regions. Once the map is built, this class knows how to
 generate the SVG for the entire map.
 
 The details depend on whether the map is a hex map or a square map. You should
-use the appropriate class instead of this one: L<Game::TextMapper::Mapper::Hex>
-or L<Game::TextMapper::Mapper::Square>.
+use the appropriate class instead of this one.
+
+=head1 SEE ALSO
+
+L<Game::TextMapper::Mapper::Hex>
+L<Game::TextMapper::Mapper::Square>
 
 =cut
 
@@ -53,17 +44,6 @@ use File::Slurper qw(read_text);
 use Encode qw(decode_utf8);
 use File::ShareDir 'dist_dir';
 
-=head1 ATTRIBUTES
-
-=head2 dist_dir
-
-You need to pass this during instantiation so that the mapper knows where to
-find files it needs to include.
-
-=cut
-
-has 'local_files';
-has 'dist_dir';
 has 'map';
 has 'regions' => sub { [] };
 has 'attributes' => sub { {} };
@@ -81,6 +61,7 @@ has 'license' => '';
 has 'other' => sub { [] };
 has 'url' => '';
 has 'offset' => sub { [] };
+has 'dist_dir';
 
 my $log = Game::TextMapper::Log->get;
 
@@ -103,7 +84,6 @@ sub example {
 0403 jungle "jungle"
 0501 mountain cave "cave"
 0502 sand "sand"
-0503 hill castle "castle"
 0205-0103-0202-0303-0402 road
 0101-0203 river
 0401-0303-0403 border
@@ -112,14 +92,6 @@ license <text>Public Domain</text>
 EOT
 }
 
-=head1 METHODS
-
-=head2 initialize($map)
-
-Call this to load a map into the mapper.
-
-=cut
-
 sub initialize {
   my ($self, $map) = @_;
   $map =~ s/&#45;/-/g; # -- are invalid in source comments...
@@ -193,16 +165,11 @@ sub process {
       } elsif (not $self->seen->{$1}) {
 	my $location = $1;
 	$self->seen->{$location} = 1;
-	my $path;
-	if (index($location, '/') == -1 and -f ($path = Mojo::File->new($self->dist_dir, $location))) {
+	my $path = Mojo::File->new($self->dist_dir, $location);
+	if (index($location, '/') == -1 and -f $path) {
 	  # without a slash, it could be a file from dist_dir
 	  $log->debug("Reading $location");
 	  $self->process(split(/\n/, decode_utf8($path->slurp())));
-	} elsif ($self->local_files and -f ($path = Mojo::File->new($location))) {
-	  # it could also be a local file in the same directory, but only if
-	  # called from the render command (which sets local_files)
-	  $log->debug("Reading $location");
-	  $self->process(split(/\n/, decode_utf8($path->slurp())));
 	} elsif ($location =~ /^https?:/) {
 	  $log->debug("Getting $location");
 	  my $ua = Mojo::UserAgent->new;
@@ -266,19 +233,21 @@ sub svg_header {
   foreach my $region (@{$self->regions}) {
     $maxz = $region->z if $region->z > $maxz;
   }
-  # These are required to calculate the viewBox for the SVG. Min and max X are
-  # what you would expect. Min and max Y are different, however, since we want
-  # to count all the rows on all the levels, plus an extra separator between
-  # them. Thus, min y is the min y of the first level, and max y is the min y of
-  # the first level + 1 for every level beyond the first, + all the rows for
-  # each level.
+  # these are required to calculate the viewBox for the SVG
   my $min_x_overall;
   my $max_x_overall;
   my $min_y_overall;
-  my $max_y_overall;
+  # $max_y_overall is the last row of the SVG with all the levels: if there is
+  # just one hex, 010100, then the last row shown on the SVG is 0 (the first
+  # one); if there are two hexes beneath each other, 010100 and 010101, then the
+  # last row shown on the SVG is 2 (y=0 is for z=0, y=1 is the space between
+  # levels, and y=2 is for z=1); note that this would be the same if the two
+  # hexes were 020200 and 020202!
+  my $max_y_overall = 0;
   for my $z (0 .. $maxz) {
     my ($minx, $miny, $maxx, $maxy);
-    $self->offset->[$z] = $max_y_overall // 0;
+    $max_y_overall += 1 if $z > 0;
+    $self->offset->[$z] = $max_y_overall;
     foreach my $region (@{$self->regions}) {
       next unless $region->z == $z;
       $minx = $region->x unless defined $minx and $minx <= $region->x;
@@ -288,10 +257,8 @@ sub svg_header {
     }
     $min_x_overall = $minx unless defined $min_x_overall and $minx >= $min_x_overall;
     $max_x_overall = $maxx unless defined $min_y_overall and $maxx <= $max_x_overall;;
-    $min_y_overall = $miny unless defined $min_y_overall; # first row of the first level
-    $max_y_overall = $miny unless defined $max_y_overall; # also (!) first row of the first level
-    $max_y_overall += 1 if $z > 0; # plus a separator row for every extra level
-    $max_y_overall += 1 + $maxy - $miny; # plus the number of rows for every level
+    $min_y_overall = $miny unless defined $min_y_overall;
+    $max_y_overall += 1 + $maxy - $miny;
   }
   my ($vx1, $vy1, $vx2, $vy2) = $self->viewbox($min_x_overall, $min_y_overall, $max_x_overall, $max_y_overall);
   my ($width, $height) = ($vx2 - $vx1, $vy2 - $vy1);
@@ -304,7 +271,7 @@ sub svg_defs {
   my ($self) = @_;
   # All the definitions are included by default.
   my $doc = "  <defs>\n";
-  $doc .= "    " . join("\n    ", @{$self->defs}, "") if @{$self->defs};
+  $doc .= "    " . join("\n    ", @{$self->defs}) if @{$self->defs};
   # collect region types from attributess and paths in case the sets don't overlap
   my %types = ();
   foreach my $region (@{$self->regions}) {
@@ -423,12 +390,6 @@ sub svg_labels {
   return $doc;
 }
 
-=head2 svg()
-
-This method generates the SVG once the map is initialized.
-
-=cut
-
 sub svg {
   my ($self) = @_;
 
@@ -461,12 +422,4 @@ sub svg {
   return $doc;
 }
 
-=head1 SEE ALSO
-
-L<Game::TextMapper::Mapper::Hex> is for hex maps.
-
-L<Game::TextMapper::Mapper::Square> is for square maps.
-
-=cut
-
 1;
