@@ -12,18 +12,12 @@
 #
 # You should have received a copy of the GNU Affero General Public License along
 # with this program. If not, see <http://www.gnu.org/licenses/>.
-
 =encoding utf8
 
 =head1 NAME
 
 Game::TextMapper::Smale - generate fantasy wilderness maps
 
-=head1 SYNOPSIS
-
-    my $text = Game::TextMapper::Smale->new
-        ->generate_map($width, $height, $bw);
-
 =head1 DESCRIPTION
 
 This generates a wilderness map based on the algorithm by Erin D. Smale. See the
@@ -31,24 +25,12 @@ blog posts at L<http://www.welshpiper.com/hex-based-campaign-design-part-1/> and
 L<http://www.welshpiper.com/hex-based-campaign-design-part-2/> for more
 information.
 
-Generally speaking, the idea is that the algorithm picks a random terrain for a
-hex in the middle of the map. Based on that, the surrounding hexes a bit further
-away are picked, and finally the remaining hexes are picked. This is why the
-maps vary so drastically in terrain distribution.
-
-=head1 METHODS
-
-Note that this module acts as a class with the C<generate_map> method, but none
-of the other subroutines defined are actual methods. They don't take a C<$self>
-argument.
-
 =cut
 
 package Game::TextMapper::Smale;
 use Game::TextMapper::Log;
 use Game::TextMapper::Point;
 use Modern::Perl '2018';
-use Mojo::Base -base;
 
 my $log = Game::TextMapper::Log->get;
 
@@ -231,7 +213,7 @@ sub place_major {
   $x += $hex->[0];
   $y += $hex->[1];
   my $coordinates = Game::TextMapper::Point::coord($x, $y);
-  my $primary = $reverse_lookup{$world{$coordinates}} || "";
+  my $primary = $reverse_lookup{$world{$coordinates}};
   my ($color, $terrain) = split(' ', $world{$coordinates}, 2);
   if ($encounter eq 'settlement') {
     if ($primary eq 'plains') {
@@ -411,7 +393,6 @@ sub agriculture {
       my $coordinates = Game::TextMapper::Point::coord($x, $y);
       if ($world{$coordinates}) {
 	my ($color, $terrain) = split(' ', $world{$coordinates}, 2);
-        next unless $reverse_lookup{$world{$coordinates}};
 	$log->debug("  $coordinates is " . $world{$coordinates} . " ie. " . $reverse_lookup{$world{$coordinates}});
 	if ($reverse_lookup{$world{$coordinates}} eq 'plains') {
 	  $log->debug("   $coordinates is a candidate");
@@ -426,18 +407,11 @@ sub agriculture {
   }
 }
 
-=head2 generate_map WIDTH, HEIGHT, BW
-
-WIDTH and HEIGHT default to 20×10.
-
-BW stands for "black & white", i.e. a true value skips background colours.
-
-=cut
-
 sub generate_map {
-  my ($self, $width, $height, $bw) = @_;
-  $width = 20 if not defined $width or $width < 1;
-  $height = 10 if not defined $height or $height < 1;
+  my ($bw, $width, $height) = @_;
+  $width = 20 if not defined $width or $width < 1 or $width > 100;
+  $height = 10 if not defined $height or $height < 1 or $height > 100;
+
   my $seeds;
   for (my $y = 1; $y < $height + 3; $y += 5) {
     for (my $x = 1; $x < $width + 3; $x += 5) {
@@ -455,10 +429,11 @@ sub generate_map {
 
   # delete extra hexes we generated to fill the gaps
   for my $coordinates (keys %world) {
-    $coordinates =~ /(-?\d\d)(-?\d\d)/ or $coordinates =~ /(-?\d\d+)\.(-?\d\d+)/;
+    $coordinates =~ /(-?\d\d)(-?\d\d)/;
     delete $world{$coordinates} if $1 < 1 or $2 < 1;
     delete $world{$coordinates} if $1 > $width or $2 > $height;
   }
+
   if ($bw) {
     for my $coordinates (keys %world) {
       my ($color, $rest) = split(' ', $world{$coordinates}, 2);
@@ -474,17 +449,4 @@ sub generate_map {
     . "include gnomeyland.txt\n";
 }
 
-=head1 SEE ALSO
-
-Erin D. Smale described this algorithm in two famous blog posts:
-L<http://www.welshpiper.com/hex-based-campaign-design-part-1/> and
-L<http://www.welshpiper.com/hex-based-campaign-design-part-2/>.
-
-The map itself uses the I<Gnomeyland> icons by Gregory B. MacKenzie. These are
-licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported License.
-To view a copy of this license, visit
-L<http://creativecommons.org/licenses/by-sa/3.0/>.
-
-=cut
-
 1;
